"use strict";

exports.__esModule = true;
exports.partialLodashMock = partialLodashMock;
function partialLodashMock() {
  return {
    debounce: (fn, delay) => {
      let timeoutId;
      function mockDebounce(...args) {
        clearTimeout(timeoutId);
        timeoutId = setTimeout(() => fn(...args), delay);
        return;
      }
      mockDebounce.cancel = () => clearTimeout(timeoutId);
      return mockDebounce;
    },
    throttle: (fn, delay) => {
      let timeoutId = null;
      const throttledFunction = function mockThrottle(...args) {
        if (timeoutId !== null) {
          return;
        }
        timeoutId = setTimeout(() => {
          fn(...args);
          timeoutId = null;
        }, delay);
      };
      throttledFunction.cancel = () => {
        if (timeoutId !== null) {
          clearTimeout(timeoutId);
          timeoutId = null;
        }
      };
      return throttledFunction;
    }
  };
}
//# sourceMappingURL=partialLodashMock.js.map