"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.PromisesSequence = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _events = require("events");
class PromisesSequence {
  constructor() {
    (0, _defineProperty2.default)(this, "fns", []);
    (0, _defineProperty2.default)(this, "events", new _events.EventEmitter());
    (0, _defineProperty2.default)(this, "state", {
      running: false
    });
  }
  async enqueue(fn) {
    const {
      fns,
      events,
      state
    } = this;
    fns.push(fn);
    if (state.running) {
      return new Promise(resolve => {
        events.once('success', resolve);
      });
    }
    events.emit('start');
    state.running = true;
    let next;
    try {
      while (fns.length) {
        next = fns.shift();
        await (next == null ? void 0 : next());
      }
      events.emit('success');
      events.emit('done');
      state.running = false;
    } catch (e) {
      if (next) {
        fns.unshift(next);
      }
      events.emit('done');
      state.running = false;
      throw e;
    }
  }
  clear() {
    this.fns.splice(0);
  }
}
exports.PromisesSequence = PromisesSequence;
//# sourceMappingURL=PromisesSequence.js.map