import { action, computed, makeObservable, observable } from 'mobx';
export class WidgetsFormState {
    constructor() {
        this._widgets = observable.map(new Map());
        makeObservable(this, {
            registerWidget: action.bound,
            isDirty: computed,
        });
    }
    registerWidget(key, state) {
        this._widgets.set(key, state);
    }
    async validate() {
        const data = {};
        let isValid = true;
        for (const [key, widget] of this._widgets.entries()) {
            const widgetData = await widget.validate();
            if (!widgetData.isValid) {
                isValid = false;
                return { isValid: false, data: {} };
            }
            data[key] = widgetData.values;
        }
        return { isValid, data };
    }
    get isDirty() {
        return [...this._widgets.values()].some((w) => w.isDirty);
    }
    get(key) {
        return this._widgets.get(key);
    }
}
//# sourceMappingURL=WidgetsFormState.js.map