import { getComponentSettings, getGlobalSettings, setComponentSettings, setGlobalSettings, } from '@wix/bex-utils/@wix/ambassador-app-settings-v1-settings/http';
import { merge } from 'lodash';
export function appSettingsMocks(whenRequest) {
    const db = {
        global: {
            settings: {},
        },
        components: {},
    };
    return [
        whenRequest(getGlobalSettings)
            .reply(200, async (data) => {
            const { host } = data;
            return {
                host,
                settings: {
                    ...db.global.settings,
                },
            };
        })
            .persist(),
        whenRequest(getComponentSettings)
            .reply(200, async (data) => {
            const { host, componentId } = data;
            if (componentId == null) {
                throw new Error(`getComponentSettings: Invalid \`componentId\`, received: "${componentId}"`);
            }
            return {
                host,
                settings: {
                    ...db.components[componentId]?.settings,
                },
            };
        })
            .persist(),
        whenRequest(setGlobalSettings)
            .reply(200, async (data) => {
            const { host, settings } = data;
            merge(db.global.settings, settings);
            return {
                host,
                settings,
            };
        })
            .persist(),
        whenRequest(setComponentSettings)
            .reply(200, async (data) => {
            const { host, settings, componentId } = data;
            if (componentId == null) {
                throw new Error(`setComponentSettings: Invalid \`componentId\`, received: "${componentId}"`);
            }
            const component = (db.components[componentId] ??= { settings: {} });
            merge(component.settings, settings);
            return {
                host,
                settings,
            };
        })
            .persist(),
    ];
}
//# sourceMappingURL=appSettingsMocks.js.map