import { ConditionalModalState } from './ConditionalModalState';
import { CustomFieldModalBiReporter } from './CustomFieldModalBiReporter';
import { FieldState, FormState } from 'formstate';
import { ToggleState } from './ToggleState';
import { CustomFieldOrigin, CustomFieldType, ReportBI } from '../services';
import { DataExtensionState } from './DataExtensionState';
import { ToastConfig } from '../model';
export interface CustomFieldStateParams {
    dataExtensionState: DataExtensionState;
    reportBi: ReportBI;
}
export interface CustomFieldForm {
    type?: CustomFieldType;
    key: string;
    name: string;
    readApps: boolean;
    readUsers: boolean;
    writeApps: boolean;
    writeUsers: boolean;
    containsPii: boolean;
}
export declare class CustomFieldModalState {
    dataExtensionState: DataExtensionState;
    customFieldModalBiReporter: CustomFieldModalBiReporter;
    settings: ToggleState;
    errorToastConfig: ToastConfig | undefined;
    mode: 'new' | 'edit';
    origin: CustomFieldOrigin;
    customOrigin?: string;
    reportBi: ReportBI;
    modalState: ConditionalModalState<unknown>;
    form: FormState<{
        type: FieldState<CustomFieldType | undefined>;
        name: FieldState<string>;
        key: FieldState<string>;
        readUsers: FieldState<boolean>;
        readApps: FieldState<boolean>;
        writeUsers: FieldState<boolean>;
        writeApps: FieldState<boolean>;
        containsPii: FieldState<boolean>;
    }>;
    disabledFields: {
        permissions: {
            read: {
                apps: boolean;
                users: boolean;
            };
            write: {
                apps: boolean;
                users: boolean;
            };
        };
    };
    constructor(params: CustomFieldStateParams);
    onConfirmError(e: unknown): void;
    reset(): void;
    enableAutoValidation(): void;
    disableAutoValidation(): void;
    getFormValues(): CustomFieldForm;
    setFormValues(form: CustomFieldForm, params?: {
        disablePermissionCheckboxes: boolean;
    }): void;
    setDefaultPermissions(): void;
    setKeyFieldIfNotDirty: (value: string) => void;
    private containsKey;
    generateUniqueKey(name: string): string;
    get t(): import("../services").Translate;
    get isSubmitting(): boolean;
    get namePlaceholder(): string;
}
//# sourceMappingURL=CustomFieldModalState.d.ts.map